import asyncio

import aiohttp

from common.exceptions import *
from common.headers import random_desktop_headers
from common.bing_urls import bing_geos
from parsing.bing_result_parser import parse_html

BING_DEFAULT_URL = 'http://www.bing.com/search?q={}&count={}'

async def bing_request(url, proxy):
    async with aiohttp.ClientSession() as client:
        try:
            async with client.get(url, headers=random_desktop_headers(), proxy=proxy, timeout=60) as response:
                html = await response.text()
                return {'html': html, 'status': response.status, 'error': None}
        except aiohttp.ClientError as err:
            return {'error': err}


def build_bing_url(geo, keyword, number):
    keyword = keyword.replace(' ', '+')
    if geo:
        url = bing_geos.get(geo, BING_DEFAULT_URL)
        return url.format(keyword, number)
    else:
        return BING_DEFAULT_URL.format(keyword, number)


def unpack_data(data_dict):
    keyword, geo, number = data_dict.get('keyword'), data_dict.get('geo'), data_dict.get('number', 50)
    proxy = data_dict.get('proxy')
    if proxy:
        proxy = 'http://{}'.format(proxy)
    return keyword, geo, number, proxy


async def bing_gather_results(data):
    result_dict = dict()
    keyword, geo, number, proxy = unpack_data(data)
    try:
        bing_url = build_bing_url(geo, keyword, number)
        html_result = await bing_request(bing_url, proxy)
        results = parse_html(html_result['html'])
        result_dict['results'] = results
        result_dict['keyword'] = keyword
        result_dict['geo'] = geo
        return result_dict
    except Exception as err:
        await asyncio.sleep(0)
        return {'error': str(err), 'keyword': keyword, 'geo': geo, 'proxy': proxy}



from lxml import html as lh
from bs4 import BeautifulSoup

def parse_html(html):
    results = []
    lxml_doc = lh.fromstring(html)
    result_blocks = lxml_doc.cssselect('li.serp-item')
    rank = 1
    for res in result_blocks:
        result = attributes(res)
        if result:
            result.update({'rank': rank})
            results.append(result)
            rank += 1
    return results


def attributes(result_block):
    html = lh.tostring(result_block).decode('utf-8', errors='ignore')
    soup = BeautifulSoup(html, 'lxml')

    link = soup.find('a', href=True)
    if link:
        link = link['href']

    title = soup.find('h2')
    if title:
        title = title.get_text()

    description = soup.find('div', {'class': 'organic__content-wrapper'})
    if description:
        description = description.get_text()

    if not link.startswith('//'):
        return {'url': link, 'title': title, 'description': description}
    else:
        return None


bing_geos = {
    'uk': 'http://www.bing.com/search?q={}&cc=GB&count={}',
    'us': 'http://www.bing.com/search?q={}&cc=US&count={}',
    'tr': 'http://www.bing.com/search?q={}&cc=TR&count={}',
    'tw': 'http://www.bing.com/search?q={}&cc=TW&count={}',
    'ch': 'http://www.bing.com/search?q={}&cc=CH&count={}',
    'se': 'http://www.bing.com/search?q={}&cc=SE&count={}',
    'es': 'http://www.bing.com/search?q={}&cc=ES&count={}',
    'za': 'http://www.bing.com/search?q={}&cc=ZA&count={}',
    'sa': 'http://www.bing.com/search?q={}&cc=SA&count={}',
    'ru': 'http://www.bing.com/search?q={}&cc=RU&count={}',
    'ph': 'http://www.bing.com/search?q={}&cc=PH&count={}',
    'pt': 'http://www.bing.com/search?q={}&cc=PT&count={}',
    'pl': 'http://www.bing.com/search?q={}&cc=PL&count={}',
    'cn': 'http://www.bing.com/search?q={}&cc=CN&count={}',
    'no': 'http://www.bing.com/search?q={}&cc=NO&count={}',
    'nz': 'http://www.bing.com/search?q={}&cc=NZ&count={}',
    'nl': 'http://www.bing.com/search?q={}&cc=NL&count={}',
    'mx': 'http://www.bing.com/search?q={}&cc=MX&count={}',
    'my': 'http://www.bing.com/search?q={}&cc=MY&count={}',
    'kr': 'http://www.bing.com/search?q={}&cc=KR&count={}',
    'j[': 'http://www.bing.com/search?q={}&cc=JP&count={}',
    'it': 'http://www.bing.com/search?q={}&cc=IT&count={}',
    'id': 'http://www.bing.com/search?q={}&cc=ID&count={}',
    'in': 'http://www.bing.com/search?q={}&cc=IN&count={}',
    'hk': 'http://www.bing.com/search?q={}&cc=HK&count={}',
    'de': 'http://www.bing.com/search?q={}&cc=DE&count={}',
    'fr': 'http://www.bing.com/search?q={}&cc=FR&count={}',
    'fi': 'http://www.bing.com/search?q={}&cc=FI&count={}',
    'dk': 'http://www.bing.com/search?q={}&cc=DK&count={}',
    'cl': 'http://www.bing.com/search?q={}&cc=CL&count={}',
    'ca': 'http://www.bing.com/search?q={}&cc=CA&count={}',
    'br': 'http://www.bing.com/search?q={}&cc=BR&count={}',
    'be': 'http://www.bing.com/search?q={}&cc=BE&count={}',
    'at': 'http://www.bing.com/search?q={}&cc=AT&count={}',
    'au': 'http://www.bing.com/search?q={}&cc=AU&count={}',
    'ar': 'http://www.bing.com/search?q={}&cc=AR&count={}'
}



yandex_geos = {
    'GLOBAL': 'https://yandex.com/search/?text={}&lr={}&numdoc={}',
    'RU': 'https://yandex.ru/search/?text={}&lr={}&numdoc={}',
    'UA': 'https://yandex.ua/search/?text={}&lr={}&numdoc={}',
    'BY': 'https://yandex.by/search/?text={}&lr={}&numdoc={}',
    'KZ': 'https://yandex.kz/search/?text={}&lr={}&numdoc={}',
    'TR': 'https://yandex.tr/search/?text={}&lr={}&numdoc={}'
}


import asyncio

from aiohttp import web

from scraping.google_scraping import google_gather_results
from scraping.bing_scraping import bing_gather_results
from scraping.yandex_scraping import yandex_gather_results
from schemas.google_schemas import GoogleSingleItem


class SearchScraper:

    def __init__(self):

        try:
            import uvloop
            asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
        except ImportError:
            pass

        self.loop = asyncio.get_event_loop()
        self.google_schema = GoogleSingleItem()

    async def scrape_google_single_keyword(self, request):
        data = await request.json()
        input_data, errors = self.google_schema.load(data)
        if errors:
            return web.json_response(errors, status=400)
        results = await google_gather_results(input_data)
        if 'error' in results:
            return web.json_response(results, status=400)
        return web.json_response(results, status=200)

    async def scrape_bing_single_keyword(self, request):
        data = await request.json()
        input_data, errors = self.google_schema.load(data)
        if errors:
            return web.json_response(errors, status=400)
        results = await bing_gather_results(input_data)
        if 'error' in results:
            return web.json_response(results, status=400)
        return web.json_response(results, status=200)

    async def scrape_yandex_single_keyword(self, request):
        data = await request.json()
        input_data, errors = self.google_schema.load(data)
        if errors:
            return web.json_response(errors, status=400)
        results = await yandex_gather_results(input_data)
        if 'error' in results:
            return web.json_response(results, status=400)
        return web.json_response(results, status=200)

    def run_server(self):
        app = web.Application(loop=self.loop)
        app.router.add_post('/google-scrape', self.scrape_google_single_keyword)
        app.router.add_post('/bing-scrape', self.scrape_bing_single_keyword)
        app.router.add_post('/yandex-scrape', self.scrape_yandex_single_keyword)
        web.run_app(app, host='127.0.0.1', port=8080)


from marshmallow import Schema, fields


class GoogleSingleItem(Schema):
    keyword = fields.String(required=True)
    geo = fields.String(required=False)
    proxy = fields.String(required=False)
    number = fields.Int(required=False)
    lr = fields.Int(required=False)



import asyncio

import aiohttp

from common.exceptions import *
from common.headers import random_desktop_headers
from common.google_urls import google_geos
from parsing.google_result_parser import parse_html

DEFAULT_GOOGLE_URL = 'https://www.google.com/search?q={}&num={}&hl=en'

async def google_request(url, proxy):
    async with aiohttp.ClientSession() as client:
        try:
            async with client.get(url, headers=random_desktop_headers(), proxy=proxy, timeout=60) as response:
                html = await response.text()
                return {'html': html, 'status': response.status}
        except aiohttp.ClientError as err:
            return {'error': err}


def build_google_url(geo, keyword, number):
    keyword = keyword.replace(' ', '+')
    if geo:
        url = google_geos.get(geo, DEFAULT_GOOGLE_URL)
        return url.format(keyword, number)
    else:
        return DEFAULT_GOOGLE_URL.format(keyword, number)


def unpack_data(data_dict):
    keyword, geo, number = data_dict.get('keyword'), data_dict.get('geo'), data_dict.get('number', '100')
    proxy = data_dict.get('proxy')
    if not keyword:
        raise NoKeywordProvided('No keyword was provided')
    if proxy:
        proxy = 'http://{}'.format(proxy)
    return keyword, geo, number, proxy


async def google_gather_results(data):
    result_dict = dict()
    keyword, geo, number, proxy = unpack_data(data)
    try:
        google_url = build_google_url(geo, keyword, number)
        html_result = await google_request(google_url, proxy)
        if html_result.get('error'):
            return {'keyword': keyword, 'geo': geo, 'proxy': proxy, 'error': 'Client error - likely connectivity issue'}
        if html_result.get('status') > 499:
            return {'keyword': keyword, 'geo': geo, 'proxy': proxy, 'error': 'Request rejected by Google'}
        results = parse_html(html_result['html'])
        result_dict['results'] = results
        result_dict['keyword'] = keyword
        result_dict['geo'] = geo
        return result_dict
    except Exception as err:
        await asyncio.sleep(0)
        return {'error': str(err), 'keyword': keyword, 'geo': geo, 'proxy': proxy}



from setuptools import setup

setup(name='searchscrapeserver',
      version='0.2.5.1',
      description='Server implementation scraping a number of popular servers',
      url='https://github.com/EdmundMartin/SearchScraperAPI',
      download_url = 'https://github.com/EdmundMartin/SearchScraperAPI/archive/0.2.3.tar.gz',
      author='Edmund Martin',
      author_email='edmartin101@googlemail.com',
      license='GPL-3.0',
      packages=['searchscrapeserver'],
      pymodules=['server','parsing'],
      install_requires=[
      	'aiohttp',
      	'lxml',
      	'bs4',
      	'marshmallow',
      ],
      zip_safe=False)